Package: sharpshootR
Type: Package
Encoding: UTF-8
Title: A Soil Survey Toolkit
Description: A collection of data processing, visualization, and export functions to support soil survey operations. Many of the functions build on the `SoilProfileCollection` S4 class provided by the aqp package, extending baseline visualization to more elaborate depictions in the context of spatial and taxonomic data. While this package is primarily developed by and for the USDA-NRCS, in support of the National Cooperative Soil Survey, the authors strive for generalization sufficient to support any soil survey operation. Many of the included functions are used by the SoilWeb suite of websites and movile applications. These functions are provided here, with additional documentation, to enable others to replicate high quality versions of these figures for their own purposes.
Version: 2.5
Authors@R: c(
  person(given="Dylan", family="Beaudette", role = c("cre", "aut"), email = "dylan.beaudette@usda.gov"), 
  person(given="Jay", family="Skovlin", role = c("aut")), 
  person(given="Stephen", family="Roecker", role = c("aut")), 
  person(given="Andrew", family="Brown", role = c("aut")), 
  person(given="USDA-NRCS Soil Survey Staff", family="", role = c("ctb"))
  )
Maintainer: Dylan Beaudette <dylan.beaudette@usda.gov>
LazyLoad: yes
LazyData: true
License: GPL (>= 3)
Repository: CRAN
URL: https://github.com/ncss-tech/sharpshootR
BugReports: https://github.com/ncss-tech/sharpshootR/issues
Suggests: MASS, spdep, circlize, rvest, xml2, terra, raster,
        exactextractr, httr, jsonlite, igraph, dendextend, testthat,
        hydromad (>= 0.9.27), latticeExtra, farver, venn, gower,
        daymetr, elevatr, Evapotranspiration, zoo, SoilTaxonomy, sf,
        Hmisc, knitr, rmarkdown, vegan
Depends: R (>= 4.1.0)
Imports: grDevices, graphics, methods, stats, utils, aqp, ape, soilDB,
        cluster, lattice, reshape2, scales, circular, RColorBrewer,
        plyr, digest, e1071, stringi, curl, grid
Additional_repositories: https://hydromad.github.io
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 17:50:37 UTC; Dylan.Beaudette
Author: Dylan Beaudette [cre, aut],
  Jay Skovlin [aut],
  Stephen Roecker [aut],
  Andrew Brown [aut],
  USDA-NRCS Soil Survey Staff [ctb]
Date/Publication: 2026-05-04 18:40:02 UTC
Built: R 4.6.0; ; 2026-05-04 23:59:44 UTC; unix
