
importFrom("grDevices", "colorRampPalette", "grey", "col2rgb", "rgb")

importFrom("graphics", "abline", "arrows", "axis", "image", "layout",
             "legend", "lines", "mtext", "par", "points", "rect",
             "segments", "strheight", "strwidth", "text", "grid", "hist", "box", "boxplot", "title",
           "barplot", "bxp", "contour", "polygon", "matplot")

importFrom("methods", "as", "slot")

importFrom("stats", "as.dendrogram", "as.formula", "as.hclust",
             "complete.cases", "cutree", "density", "lm", "median",
             "na.omit", "predict", "quantile", "dnorm", "sd", "update",
           "approxfun", "splinefun", "as.ts", "aggregate", "rmultinom")

importFrom("utils", "URLencode", "download.file", "read.csv", "setTxtProgressBar", 
           "txtProgressBar", "packageVersion", "combn")


importFrom(ape,
       .PlotPhyloEnv,
       as.phylo,
       is.ultrametric,
       tiplabels,
       rotateConstr
)

importFrom(aqp,
           plotProfileDendrogram,
           plot,
           hzDistinctnessCodeToOffset,
           fixOverlap,
           idname,
           plotSPC,
           profile_id,
           site,
           huePosition,
           invertLabelColor,
           parseMunsell,
           mixMunsell,
           'depths<-',
           'site<-',
           slab,
           shannonEntropy,
           hzdesgnname,
           'hzdesgnname<-'
)

importFrom(curl,
  has_internet
)

importFrom(stringi,
  stri_match_first_regex,
  stri_pad_left,
  stri_replace_all_fixed,
  stri_sub,
  stri_length
)

importFrom(cluster,
  daisy,
  diana
)

importFrom(grid,
  grid.text,
  unit,
  gpar
)

importFrom(lattice,
  levelplot,
  panel.barchart,
  panel.levelplot,
  panel.segments,
  panel.rect,
  panel.arrows,
  panel.grid,
  strip.custom,
  panel.abline,
  trellis.par.set,
  trellis.par.get,
  barchart,
  simpleKey,
  yscale.components.default
)

importFrom(digest,
  digest
)

importFrom(e1071,
  skewness
)

importFrom(soilDB,
  format_SQL_in_statement,
  SDA_query,
  waterDayYear
)


importFrom(reshape2,
  dcast,
  melt
)
	
importFrom(scales, 
  alpha,
  rescale
)

importFrom(circular, 
  circular,
  mean.circular,
  rho.circular,
  plot.circular,
  density.circular,
  axis.circular,
  arrows.circular,
  rayleigh.test
)

importFrom(RColorBrewer, 
  brewer.pal
)

importFrom(plyr,
  dlply,
  ldply
)


exportPattern("^[^\\.]")
