% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moistureStateProportions.R
\name{moistureStateProportions}
\alias{moistureStateProportions}
\title{Compute moisture state proportions}
\usage{
moistureStateProportions(x, id = "compname", step = c("month", "week", "doy"))
}
\arguments{
\item{x}{\code{data.frame} created by \code{\link[=dailyWB]{dailyWB()}} or \code{\link[=dailyWB_SSURGO]{dailyWB_SSURGO()}}}

\item{id}{character, column name identifying sites, components, or soil series}

\item{step}{time step, one of 'month', 'week', or 'doy'}
}
\value{
\code{data.frame}
}
\description{
Compute moisture state proportions
}
